import random # Pour les tirages aleatoires
import pygame # Le module Pygame
import pygame.freetype # Pour afficher du texte
pygame.init() # initialisation de Pygame

# L'image de fond
largeur, hauteur = 800, 600

# On définit la taille de la fenêtre
screen = pygame.display.set_mode((largeur, hauteur))

# On met le nom de la fenêtre
pygame.display.set_caption("Serpent")

# On récupère l'image du smiley et on obtient sa taille
smiley = pygame.image.load("smiley.png").convert_alpha()
# On va utiliser des cases de 20x20
taille_case = 20
demie_case = taille_case // 2
smiley = pygame.transform.smoothscale(smiley, (taille_case, taille_case))
x_max = largeur//taille_case - 1
y_max = hauteur//taille_case - 1

# Pour le texte
pygame.freetype.init()
police_normale = pygame.freetype.SysFont(None, 20)
police_grande = pygame.freetype.SysFont(None, 60)

# Pour controler le nombre d'images par seconde
clock = pygame.time.Clock()

def position_aleatoire():
    """
    Renvoie les coordonnées d'une case
    choisie au hasard...
    """
    x = random.randint(0, x_max)
    y = random.randint(0, y_max)
    return x, y

def afficher_texte(texte, point, couleur=(255, 255, 255),
                   alignement="haut-gauche", police=police_normale):
    texte, rect = police.render(texte, couleur)
    if alignement == "haut-gauche":
        rect.topleft = point
    elif alignement == "haut":
        rect.midtop = point
    elif alignement == "haut-droite":
        rect.topright = point
    elif alignement == "droite":
        rect.midright = point
    elif alignement == "bas-droite":
        rect.bottomright = point
    elif alignement == "bas":
        rect.midbottom = point
    elif alignement == "bas-gauche":
        rect.bottomleft = point
    elif alignement == "gauche":
        rect.midleft = point
    elif alignement == "centre":
        rect.center = point
    screen.blit(texte, rect)

def afficher_image(image, point, alignement="haut-gauche", l=-1, h=-1):
    """
    Affiche l'image aux coordonnées indiquées par le point
    et l'alignement indiqués.
    Si l ou h valent autre chose que -1,
    l'image est redimensionnée.
    Si un des deux vaut -1, il est calculé
    pour garder le ratio de l'image.
    """
    l0, h0 = image.get_size()
    if (l, h) == (-1, -1):
        l, h = l0, h0
    elif l == -1 and h != -1:
        l = round(l0*h/h0)
        image = pygame.transform.smoothscale(image, (l, h))
    elif l != -1 and h == -1:
        h = round(h0*l/l0)
        image = pygame.transform.smoothscale(image, (l, h))
    else:
        image = pygame.transform.smoothscale(image, (l, h))
    if alignement == "haut-gauche":
        rect = image.get_rect(topleft=point)
    elif alignement == "haut":
        rect = image.get_rect(midtop=point)
    elif alignement == "haut-droite":
        rect = image.get_rect(topright=point)
    elif alignement == "droite":
        rect = image.get_rect(midright=point)
    elif alignement == "bas-droite":
        rect = image.get_rect(bottomright=point)
    elif alignement == "bas":
        rect = image.get_rect(midbottom=point)
    elif alignement == "bas-gauche":
        rect = image.get_rect(bottomleft=point)
    elif alignement == "gauche":
        rect = image.get_rect(midleft=point)
    elif alignement == "centre":
        rect = image.get_rect(center=point)
    screen.blit(image, rect)

def deplacement(x, y, direction):
    """
    Renvoie la prochaine case à partir de (x, y)
    dans la direction indiquée.
    Si on sort de la fenêtre, on fait un quart de tour
    dans le sens des aiguilles d'une montre.
    N -> E -> S -> O -> N
    """
    pas_ok = True
    while pas_ok:
        if direction == "N":
            x2, y2 = x, y-1
        elif direction == "S":
            x2, y2 = x, y
        elif direction == "E":
            x2, y2 = x+1, y
        elif direction == "O":
            x2, y2 = x, y
        # On regarde si on sort du cadre
        if x2 < 0:
            direction = "N"
        elif y2 < 0:
            ...
        elif x2 > x_max:
            ...
        elif y2 > y_max:
            ...
        else:
            pas_ok = False
    return x2, y2, direction

def afficher_cercle(x, y, couleur):
    pygame.draw.circle(screen, couleur, (x*taille_case+demie_case, y*taille_case+demie_case), demie_case)

def afficher_serpent(corps):
    # On affiche des disques jaunes 
    for x, y in corps:
        afficher_cercle(x, y, (255, 255, 0))
    # On remet l'image sur la dernière
    afficher_image(smiley, (x*taille_case, y*taille_case), alignement="haut-gauche", l=-1, h=-1)

def afficher_pomme(x_pomme, y_pomme):
    afficher_cercle(x_pomme, y_pomme, (255, 0, 0))

def afficher_obstacles(obstacles):
    for x, y in obstacles:
        pygame.draw.rect(screen, (0, 255, 0), pygame.Rect(x*taille_case, y*taille_case, taille_case, taille_case))

def debut_jeu():
    corps = [(x_max//2, y_max//2)]
    direction = "E"
    x_pomme, y_pomme = position_aleatoire()
    score = 0
    return corps, direction, x_pomme, y_pomme, score

# Initialisation du jeu
debut = True
continuer = True
etape = "menu"
horloge = 0
temps_avant_mouvement = 6  # nombre d'images entre 2 mouvements

# Boucle principale
while continuer:
    # On remplit l'image de noir
    screen.fill((0, 0, 0))

    # Gestion des évènements
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            continuer = False  # Pour quitter
        elif event.type == pygame.KEYDOWN:
            if etape in ["menu", "gameover"] and horloge == 0:
                etape = "jeu"
                debut = True
            elif event.key == pygame.K_UP:  # Flèche du haut
                buffer.append("N")
            elif event.key == pygame.K_DOWN:  # Flèche du bas
                buffer.append("S")
            elif event.key == pygame.K_RIGHT:  # Flèche de droite
                buffer.append("E")
            elif event.key == pygame.K_LEFT:  # Flèche de gauche
                buffer.append("O")
    if debut and etape == "jeu":
        # On place le serpent et la première pomme
        debut = False
        score = 0
        corps = [(x_max//2, y_max//2)]
        direction = "E"
        x_pomme, y_pomme = position_aleatoire()
        obstacles = []
        buffer = []
    if etape == "jeu":
        if horloge >= temps_avant_mouvement:
            horloge = 0
            xs, ys = corps[-1]
            if len(buffer) > 0:
                direction = buffer.pop(0)  # On récupère la 1e direction indiquée
            x, y, direction = deplacement(xs, ys, direction)
            if (x, y) == (x_pomme, y_pomme):  # On attrape la pomme
                # On met une nouvelle pomme
                x_pomme, y_pomme = (x, y)
                while (x_pomme, y_pomme) == (x, y) or (x_pomme, y_pomme) in corps:
                    # On ne la met pas sur le serpent
                    x_pomme, y_pomme = position_aleatoire()
                # On rajoute un obstacle
                obst = (x, y)
                while obst == (x, y) or obst in corps or obst == (x_pomme, y_pomme) or obst in obstacles:
                    # On évite le serpent, la pomme et les autres obstacles
                    obst = position_aleatoire()
                obstacles.append(obst)
            elif (x, y) in corps:  # On touche le corps ou un obstacle
                etape = "gameover"
            else:  # Tout se passe normalement
                corps.pop(0)  # On supprime la partie du corps la plus ancienne
            corps.append((x, y))
        else:
            horloge += 1
        # Affichage
        afficher_pomme(x_pomme, y_pomme)
        afficher_serpent(corps)
    elif etape == "gameover":
        # Affichage
        afficher_pomme(x_pomme, y_pomme)
        afficher_serpent(corps)
        # Affichage des textes
        afficher_texte("Game Over", (largeur//2, hauteur//2-40), (255, 255, 255), "bas", police_grande)
        afficher_texte(f"Score : {score}", (largeur//2, hauteur//2-30), (255, 255, 255), "haut", police_normale)
    elif etape == "menu":
        afficher_texte("Appuyer sur une touche", (largeur//2, hauteur//2), (255, 255, 255), "centre", police_grande)
        
    
    # On affiche le resultat final a l'écran
    pygame.display.flip()
    # On attend le temps necessaire pour avoir un FPS constant
    clock.tick(60)

# On ferme la fenêtre
pygame.quit()